<?php

use Phinx\Migration\AbstractMigration;
use Phinx\Db\Adapter\MysqlAdapter;

final class CreateUsersAndDavTables extends AbstractMigration
{
    public function change(): void
    {
        // === Users table ===
        $users = $this->table('users', ['id' => false, 'primary_key' => ['username']]);
        $users->addColumn('username', 'string', ['limit' => 50])
              ->addColumn('digesta1', 'string', ['limit' => 255])
              ->create();

        // === Principals table (users and groups) ===
        $principals = $this->table('principals');
        $principals->addColumn('uri', 'string', ['limit' => 100])
                   ->addColumn('email', 'string', ['limit' => 255, 'null' => true])
                   ->addColumn('displayname', 'string', ['limit' => 255, 'null' => true])
                   ->addColumn('password', 'string', ['limit' => 255, 'null' => true])
                   ->addColumn('calendar_prefix', 'string', ['limit' => 20, 'null' => true])
                   ->addColumn('backend', 'string', ['limit' => 20, 'null' => true])
                   ->addIndex(['uri'], ['unique' => true])
                   ->create();

        // Foreign key principals.uri → users.username (optional but typical)
        $principals->addForeignKey('uri', 'users', 'username', ['delete'=> 'CASCADE', 'update'=> 'CASCADE'])->save();

        // === Calendars table (CalDAV) ===
        $calendars = $this->table('calendars');
        $calendars->addColumn('principaluri', 'string', ['limit' => 100])
                  ->addColumn('displayname', 'string', ['limit' => 100, 'null' => true])
                  ->addColumn('uri', 'string', ['limit' => 200])
                  ->addColumn('description', 'text', ['null' => true])
                  ->addColumn('calendarorder', 'integer', ['null' => true])
                  ->addColumn('calendarcolor', 'string', ['limit' => 10, 'null' => true])
                  ->addColumn('timezone', 'text', ['null' => true])
                  ->addColumn('components', 'string', ['limit' => 20, 'null' => true])
                  ->addColumn('transparent', 'boolean', ['null' => true])
                  ->addIndex(['principaluri', 'uri'], ['unique' => true])
                  ->create();

        $calendars->addForeignKey('principaluri', 'principals', 'uri', ['delete'=> 'CASCADE', 'update'=> 'CASCADE'])->save();

        // === Calendar Objects table (CalDAV) ===
        $calendarObjects = $this->table('calendarobjects');
        $calendarObjects->addColumn('calendarid', 'integer')
                        ->addColumn('uri', 'string', ['limit' => 200])
                        ->addColumn('calendardata', 'binary', ['limit' => MysqlAdapter::BLOB_MEDIUM, 'null' => true])
                        ->addColumn('lastmodified', 'integer', ['null' => true])
                        ->addColumn('etag', 'string', ['limit' => 32, 'null' => true])
                        ->addColumn('size', 'integer', ['null' => true])
                        ->addColumn('componenttype', 'string', ['limit' => 8, 'null' => true])
                        ->addColumn('firstoccurence', 'integer', ['null' => true])
                        ->addColumn('lastoccurence', 'integer', ['null' => true])
                        ->addColumn('uid', 'string', ['limit' => 200, 'null' => true])
                        ->addIndex(['calendarid', 'uri'], ['unique' => true])
                        ->create();

        $calendarObjects->addForeignKey('calendarid', 'calendars', 'id', ['delete'=> 'CASCADE', 'update'=> 'CASCADE'])->save();

        // === Addressbooks table (CardDAV) ===
        $addressbooks = $this->table('addressbooks');
        $addressbooks->addColumn('principaluri', 'string', ['limit' => 100])
                     ->addColumn('displayname', 'string', ['limit' => 100, 'null' => true])
                     ->addColumn('uri', 'string', ['limit' => 200])
                     ->addColumn('description', 'text', ['null' => true])
                     ->addIndex(['principaluri', 'uri'], ['unique' => true])
                     ->create();

        $addressbooks->addForeignKey('principaluri', 'principals', 'uri', ['delete'=> 'CASCADE', 'update'=> 'CASCADE'])->save();

        // === Addressbook Changes table (CardDAV) ===
        $addressbookChanges = $this->table('addressbookchanges');
        $addressbookChanges->addColumn('addressbookid', 'integer')
                           ->addColumn('uri', 'string', ['limit' => 200, 'null' => true])
                           ->addColumn('synctoken', 'integer', ['null' => true])
                           ->create();

        $addressbookChanges->addForeignKey('addressbookid', 'addressbooks', 'id', ['delete'=> 'CASCADE', 'update'=> 'CASCADE'])->save();

        // === Addressbook Objects table (CardDAV) ===
        $addressbookObjects = $this->table('addressbookobjects');
        $addressbookObjects->addColumn('addressbookid', 'integer')
                           ->addColumn('uri', 'string', ['limit' => 200])
                           ->addColumn('carddata', 'binary', ['limit' => MysqlAdapter::BLOB_MEDIUM, 'null' => true])
                           ->addColumn('lastmodified', 'integer', ['null' => true])
                           ->addColumn('etag', 'string', ['limit' => 32, 'null' => true])
                           ->addColumn('size', 'integer', ['null' => true])
                           ->addColumn('uid', 'string', ['limit' => 200, 'null' => true])
                           ->addIndex(['addressbookid', 'uri'], ['unique' => true])
                           ->create();

        $addressbookObjects->addForeignKey('addressbookid', 'addressbooks', 'id', ['delete'=> 'CASCADE', 'update'=> 'CASCADE'])->save();

        // === Locks table (WebDAV) ===
        $locks = $this->table('locks');
        $locks->addColumn('id', 'string', ['limit' => 32])
              ->addColumn('token', 'string', ['limit' => 255, 'null' => true])
              ->addColumn('principaluri', 'string', ['limit' => 100, 'null' => true])
              ->addColumn('scope', 'string', ['limit' => 10, 'null' => true])
              ->addColumn('depth', 'string', ['limit' => 10, 'null' => true])
              ->addColumn('uri', 'string', ['limit' => 255, 'null' => true])
              ->addColumn('timeout', 'integer', ['null' => true])
              ->addColumn('created', 'integer', ['null' => true])
              ->addColumn('owner', 'text', ['null' => true])
              ->addColumn('exclusive', 'boolean', ['null' => true])
              ->addIndex(['id'], ['unique' => true])
              ->create();

        // === Calendar Shares table ===
        $calendarShares = $this->table('calendarshares');
        $calendarShares->addColumn('calendarid', 'integer')
                       ->addColumn('principaluri', 'string', ['limit' => 100])
                       ->addColumn('access', 'integer') // e.g. 1=read, 2=write
                       ->addColumn('share_access', 'integer', ['null' => true])
                       ->addColumn('summary', 'string', ['limit' => 255, 'null' => true])
                       ->addColumn('href', 'string', ['limit' => 255, 'null' => true])
                       ->addColumn('uid', 'string', ['limit' => 200, 'null' => true])
                       ->addColumn('lastupdated', 'integer', ['null' => true])
                       ->addIndex(['calendarid', 'principaluri'], ['unique' => true])
                       ->create();

        $calendarShares->addForeignKey('calendarid', 'calendars', 'id', ['delete'=> 'CASCADE', 'update'=> 'CASCADE'])->save();
        $calendarShares->addForeignKey('principaluri', 'principals', 'uri', ['delete'=> 'CASCADE', 'update'=> 'CASCADE'])->save();

        // === Addressbook Shares table ===
        $addressbookShares = $this->table('addressbookshares');
        $addressbookShares->addColumn('addressbookid', 'integer')
                          ->addColumn('principaluri', 'string', ['limit' => 100])
                          ->addColumn('access', 'integer')
                          ->addColumn('share_access', 'integer', ['null' => true])
                          ->addColumn('summary', 'string', ['limit' => 255, 'null' => true])
                          ->addColumn('href', 'string', ['limit' => 255, 'null' => true])
                          ->addColumn('uid', 'string', ['limit' => 200, 'null' => true])
                          ->addColumn('lastupdated', 'integer', ['null' => true])
                          ->addIndex(['addressbookid', 'principaluri'], ['unique' => true])
                          ->create();

        $addressbookShares->addForeignKey('addressbookid', 'addressbooks', 'id', ['delete'=> 'CASCADE', 'update'=> 'CASCADE'])->save();
        $addressbookShares->addForeignKey('principaluri', 'principals', 'uri', ['delete'=> 'CASCADE', 'update'=> 'CASCADE'])->save();
    }
}

