<?php
require_once __DIR__ . '/../vendor/autoload.php';

use WireDAV\DB\PDOProvider;

// Initiate basics
$pdo    = (new PDOProvider())->getPDO();
$method = $_SERVER['REQUEST_METHOD'];
$path   = $_GET['action'] ?? '';


// Helper Functions
function getBasicAuthUserAndPass() {
    if (!isset($_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW'])) {
        header('WWW-Authenticate: Basic realm="Admin Area"');
        header('HTTP/1.0 401 Unauthorized');
        exit('Authentication required');
    }
    return [$_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW']];
}

function isAdmin(PDO $pdo, $username, $password, $realm = 'SabreDAV') {
    $stmt = $pdo->prepare("SELECT digesta1 FROM users WHERE username = ?");
    $stmt->execute([$username]);
    $hash = $stmt->fetchColumn();
    if (!$hash) return false;

    $digest = md5("$username:$realm:$password");
    return $digest === $hash; // Only checking password; add is_admin column if you want real admin check
}

function anyUsersExist(PDO $pdo) {
    $stmt = $pdo->query("SELECT COUNT(*) FROM users");
    return $stmt->fetchColumn() > 0;
}


// Request Routing
if ($method === 'POST' && $path === 'create_user') {
    list($user, $pass) = getBasicAuthUserAndPass();

    if (anyUsersExist($pdo) && !isAdmin($pdo, $user, $pass)) {
        http_response_code(403);
        exit(json_encode(['error' => 'Forbidden: admin required']));
    }

    $input = json_decode(file_get_contents('php://input'), true);
    if (empty($input['username']) || empty($input['password'])) {
        http_response_code(400);
        exit(json_encode(['error' => 'Missing username or password']));
    }

    $newUser = $input['username'];
    $newPass = $input['password'];
    $realm = 'SabreDAV';

    $stmt = $pdo->prepare("SELECT COUNT(*) FROM users WHERE username = ?");
    $stmt->execute([$newUser]);
    if ($stmt->fetchColumn() > 0) {
        http_response_code(409);
        exit(json_encode(['error' => 'User already exists']));
    }

    $digest = md5("$newUser:$realm:$newPass");

    $stmt = $pdo->prepare("INSERT INTO users (username, digesta1) VALUES (?, ?)");
    $stmt->execute([$newUser, $digest]);

    echo json_encode(['success' => true, 'message' => "User $newUser created"]);
    exit;
}

if ($method === 'POST' && $path === 'change_password') {
    list($user, $pass) = getBasicAuthUserAndPass();

    if (!isAdmin($pdo, $user, $pass)) {
        http_response_code(403);
        exit(json_encode(['error' => 'Forbidden: admin required']));
    }

    $input = json_decode(file_get_contents('php://input'), true);
    if (empty($input['username']) || empty($input['password'])) {
        http_response_code(400);
        exit(json_encode(['error' => 'Missing username or password']));
    }

    $targetUser = $input['username'];
    $newPass = $input['password'];
    $realm = 'SabreDAV';

    $digest = md5("$targetUser:$realm:$newPass");

    $stmt = $pdo->prepare("UPDATE users SET digesta1 = ? WHERE username = ?");
    $stmt->execute([$digest, $targetUser]);

    echo json_encode(['success' => true, 'message' => "Password changed for $targetUser"]);
    exit;
}

if ($method === 'POST' && $path === 'delete_user') {
    list($user, $pass) = getBasicAuthUserAndPass();

    if (!isAdmin($pdo, $user, $pass)) {
        http_response_code(403);
        exit(json_encode(['error' => 'Forbidden: admin required']));
    }

    $input = json_decode(file_get_contents('php://input'), true);
    if (empty($input['username'])) {
        http_response_code(400);
        exit(json_encode(['error' => 'Missing username']));
    }

    $targetUser = $input['username'];

    $stmt = $pdo->prepare("DELETE FROM users WHERE username = ?");
    $stmt->execute([$targetUser]);

    echo json_encode(['success' => true, 'message' => "User $targetUser deleted"]);
    exit;
}

if ($method === 'POST' && $path === 'create_calendar') {
    list($username, $password) = getBasicAuthUserAndPass();

    // Authenticate user (no admin check, calendar owned by user)
    $stmt = $pdo->prepare("SELECT digesta1 FROM users WHERE username = ?");
    $stmt->execute([$username]);
    $hash = $stmt->fetchColumn();
    $realm = 'SabreDAV';
    if (!$hash || md5("$username:$realm:$password") !== $hash) {
        http_response_code(403);
        exit(json_encode(['error' => 'Invalid credentials']));
    }

    $input = json_decode(file_get_contents('php://input'), true);
    if (empty($input['uri']) || empty($input['displayname'])) {
        http_response_code(400);
        exit(json_encode(['error' => 'Missing uri or displayname']));
    }

    $calendarUri = $input['uri'];
    $displayName = $input['displayname'];
    $principalUri = "principals/$username";

    // Check if calendar URI already exists for user
    $stmt = $pdo->prepare("SELECT COUNT(*) FROM calendars WHERE principaluri = ? AND uri = ?");
    $stmt->execute([$principalUri, $calendarUri]);
    if ($stmt->fetchColumn() > 0) {
        http_response_code(409);
        exit(json_encode(['error' => 'Calendar URI already exists']));
    }

    $stmt = $pdo->prepare("INSERT INTO calendars (principaluri, displayname, uri) VALUES (?, ?, ?)");
    $stmt->execute([$principalUri, $displayName, $calendarUri]);

    echo json_encode(['success' => true, 'message' => "Calendar '$displayName' created"]);
    exit;
}

if ($method === 'POST' && $path === 'delete_calendar') {
    list($username, $password) = getBasicAuthUserAndPass();

    // Authenticate user
    $stmt = $pdo->prepare("SELECT digesta1 FROM users WHERE username = ?");
    $stmt->execute([$username]);
    $hash = $stmt->fetchColumn();
    $realm = 'SabreDAV';
    if (!$hash || md5("$username:$realm:$password") !== $hash) {
        http_response_code(403);
        exit(json_encode(['error' => 'Invalid credentials']));
    }

    $input = json_decode(file_get_contents('php://input'), true);
    if (empty($input['uri'])) {
        http_response_code(400);
        exit(json_encode(['error' => 'Missing calendar uri']));
    }

    $calendarUri = $input['uri'];
    $principalUri = "principals/$username";

    $stmt = $pdo->prepare("DELETE FROM calendars WHERE principaluri = ? AND uri = ?");
    $stmt->execute([$principalUri, $calendarUri]);

    if ($stmt->rowCount() === 0) {
        http_response_code(404);
        exit(json_encode(['error' => 'Calendar not found']));
    }

    echo json_encode(['success' => true, 'message' => "Calendar '$calendarUri' deleted"]);
    exit;
}


// Default fallback if no route matched
http_response_code(404);
echo json_encode(['error' => 'Unknown endpoint']);
exit;

